import numpy as np
import matplotlib.pyplot as pt
import time

def adaptive(f, a, b, N, xlim, tol):
# integrafes f from a to b using N point trapezoid rule
# xlim denotes the x limits for the plot
    x = np.linspace(a, b, N)
    y = f(x)
    pt.fill_between(x, 0, y, facecolor='g')
    pt.xlim(xlim)
    pt.draw()
    pt.show(block=False)
    time.sleep(0.1)

    A = np.trapz(y, x)

    # Find Al - Area of left half using N point trapezoid rule

    # Find Ar - Area of right half using N point trapezoid rule

    if abs((Al + Ar) - A) < tol:
        pt.fill_between(x, 0, y, facecolor='r')
        pt.draw()
        return # The area
    else:
        return # Sum of recursively computed area of left half and right

def xx2(x):
# test function
# returns x if x <= 1, x**2 otherwise
    if np.isscalar(x):
        return x if x <=1 else x**2
    else:
        x = np.asarray(x)
        return np.array([x[i] if x[i] <=1 else x[i]**2
                for i in range(x.shape[0])])

if __name__ == '__main__':
    print('integral x^2 from 0 to 1')
    pt.figure(1)
    pt.title('integral x^2 from 0 to 1')
    area = adaptive(lambda x: x**2,
            0, 1, 2, [0,1], 1.0e-4)
    print('Area =', area, '\n')

    print('integral x^3 from 0 to 1')
    pt.figure(2)
    pt.title('integral x^3 from 0 to 1')
    area = adaptive(lambda x: x**3,
            0, 1, 10, [0,1], 1.0e-7)
    print('Area =', area, '\n')

    print('integral sin(x) from 0 to pi/2')
    pt.figure(3)
    pt.title('integral sin(x) from 0 to pi/2')
    area = adaptive(np.sin,
            0, np.pi/2, 1000, [0,np.pi/2], 1.0e-10)
    print('Area =', area, '\n')

    print('integral xx2 from 0 to 2')
    pt.figure(4)
    pt.title('integral xx2 from 0 to 2')
    area = adaptive(xx2,
            0, 2, 10, [0,2], 1.0e-6)
    print('Area =', area, '\n')

    pt.show()
